@@ -28,7 +28,7 @@
 #include "scripting/toplevel/Array.h"
 #include "scripting/toplevel/ASString.h"
 
-#include "extscriptobject.h"
+#include "backends/extscriptobject.h"
 
 using namespace lightspark;
 using namespace std;
@@ -36,22 +36,22 @@ using namespace std;
 /* -- ExtIdentifier -- */
 // Constructors
 ExtIdentifier::ExtIdentifier() :
-	type(EI_STRING), strValue(""), intValue(0)
+	strValue(""), intValue(0), type(EI_STRING)
 {
 }
 ExtIdentifier::ExtIdentifier(const std::string& value) :
-	type(EI_STRING), strValue(value), intValue(0)
+	strValue(value), intValue(0), type(EI_STRING)
 {
 	stringToInt();
 }
 ExtIdentifier::ExtIdentifier(const char* value) :
-	type(EI_STRING), strValue(value), intValue(0)
+	strValue(value), intValue(0), type(EI_STRING)
 {
 	stringToInt();
 }
 
 ExtIdentifier::ExtIdentifier(int32_t value) :
-	type(EI_INT32), strValue(""), intValue(value)
+	strValue(""), intValue(value), type(EI_INT32)
 {
 }
 ExtIdentifier::ExtIdentifier(const ExtIdentifier& other)
@@ -117,13 +117,12 @@ bool ExtObject::hasProperty(const ExtIdentifier& id) const
 {
 	return properties.find(id) != properties.end();
 }
-ExtVariant* ExtObject::getProperty(const ExtIdentifier& id) const
+const ExtVariant& ExtObject::getProperty(const ExtIdentifier& id) const
 {
 	std::map<ExtIdentifier, ExtVariant>::const_iterator it = properties.find(id);
-	if(it == properties.end())
-		return NULL;
+	assert(it != properties.end());
 
-	return new ExtVariant(it->second);
+	return it->second;
 }
 void ExtObject::setProperty(const ExtIdentifier& id, const ExtVariant& value)
 {
@@ -156,35 +155,31 @@ bool ExtObject::enumerate(ExtIdentifier*** ids, uint32_t* count) const
 /* -- ExtVariant -- */
 // Constructors
 ExtVariant::ExtVariant() :
-	type(EV_VOID), strValue(""), intValue(0), doubleValue(0), booleanValue(false)
+	strValue(""), doubleValue(0), intValue(0), type(EV_VOID), booleanValue(false)
 {
 }
 ExtVariant::ExtVariant(const std::string& value) :
-	type(EV_STRING), strValue(value), intValue(0), doubleValue(0), booleanValue(false)
+	strValue(value), doubleValue(0), intValue(0), type(EV_STRING), booleanValue(false)
 {
 }
 ExtVariant::ExtVariant(const char* value) :
-	type(EV_STRING), strValue(value), intValue(0), doubleValue(0), booleanValue(false)
+	strValue(value), doubleValue(0), intValue(0), type(EV_STRING), booleanValue(false)
 {
 }
 ExtVariant::ExtVariant(int32_t value) :
-	type(EV_INT32), strValue(""), intValue(value), doubleValue(0), booleanValue(false)
+	strValue(""), doubleValue(0), intValue(value), type(EV_INT32), booleanValue(false)
 {
 }
 ExtVariant::ExtVariant(double value) :
-	type(EV_DOUBLE), strValue(""), intValue(0), doubleValue(value), booleanValue(false)
+	strValue(""), doubleValue(value), intValue(0), type(EV_DOUBLE), booleanValue(false)
 {
 }
 ExtVariant::ExtVariant(bool value) :
-	type(EV_BOOLEAN), strValue(""), intValue(0), doubleValue(0), booleanValue(value)
+	strValue(""), doubleValue(0), intValue(0), type(EV_BOOLEAN), booleanValue(value)
 {
 }
-ExtVariant::ExtVariant(const ExtVariant& other)
-{
-	*this=other;
-}
-ExtVariant::ExtVariant(_R<ASObject> other) :
-	strValue(""), intValue(0), doubleValue(0), booleanValue(false)
+ExtVariant::ExtVariant(std::map<const ASObject*, std::unique_ptr<ExtObject>>& objectsMap, _R<ASObject> other) :
+	strValue(""), doubleValue(0), intValue(0), booleanValue(false)
 {
 	switch(other->getObjectType())
 	{
@@ -205,10 +200,21 @@ ExtVariant::ExtVariant(_R<ASObject> other) :
 		type = EV_BOOLEAN;
 		break;
 	case T_ARRAY:
-		objectValue.setType(ExtObject::EO_ARRAY);
 	case T_OBJECT:
-		type = EV_OBJECT;
 		{
+			type = EV_OBJECT;
+			auto it=objectsMap.find(other.getPtr());
+			if(it!=objectsMap.end())
+			{
+				objectValue = it->second.get();
+				break;
+			}
+			objectValue = new ExtObject();
+			if(other->getObjectType()==T_ARRAY)
+				objectValue->setType(ExtObject::EO_ARRAY);
+
+			objectsMap[other.getPtr()] = move(unique_ptr<ExtObject>(objectValue));
+
 			unsigned int index = 0;
 			while((index=other->nextNameIndex(index))!=0)
 			{
@@ -216,9 +222,9 @@ ExtVariant::ExtVariant(_R<ASObject> other) :
 				_R<ASObject> nextValue=other->nextValue(index);
 
 				if(nextName->getObjectType() == T_INTEGER)
-					objectValue.setProperty(nextName->toInt(), nextValue);
+					objectValue->setProperty(nextName->toInt(), ExtVariant(objectsMap, nextValue));
 				else
-					objectValue.setProperty(nextName->toString().raw_buf(), nextValue);
+					objectValue->setProperty(nextName->toString().raw_buf(), ExtVariant(objectsMap, nextValue));
 			}
 		}
 		break;
@@ -232,41 +238,8 @@ ExtVariant::ExtVariant(_R<ASObject> other) :
 	}
 }
 
-ExtVariant& ExtVariant::operator=(const ExtVariant& other)
-{
-	type = other.getType();
-
-	switch(type)
-	{
-	case EV_STRING:
-		strValue = other.getString();
-		break;
-	case EV_INT32:
-		intValue = other.getInt();
-		break;
-	case EV_DOUBLE:
-		doubleValue = other.getDouble();
-		break;
-	case EV_BOOLEAN:
-		booleanValue = other.getBoolean();
-		break;
-	case EV_OBJECT:
-		{
-			ExtObject* tmpObj=other.getObject();
-			objectValue = *tmpObj;
-			delete tmpObj;
-		}
-		break;
-	case EV_NULL:
-	case EV_VOID:
-		break;
-	}
-
-	return *this;
-}
-
 // Conversion to ASObject
-ASObject* ExtVariant::getASObject() const
+ASObject* ExtVariant::getASObject(std::map<const lightspark::ExtObject*, lightspark::ASObject*>& objectsMap) const
 {
 	ASObject* asobj;
 	switch(getType())
@@ -286,62 +259,66 @@ ASObject* ExtVariant::getASObject() const
 	case EV_OBJECT:
 		{
 			ExtObject* objValue = getObject();
+			auto it=objectsMap.find(objValue);
+			if(it!=objectsMap.end())
+			{
+				it->second->incRef();
+				return it->second;
+			}
 
-			ExtVariant* property;
 			uint32_t count;
 
 			// We are converting an array, so lets set indexes
 			if(objValue->getType() == ExtObject::EO_ARRAY)
 			{
 				asobj = Class<Array>::getInstanceS();
+				objectsMap[objValue] = asobj;
+
 				count = objValue->getLength();
 				static_cast<Array*>(asobj)->resize(count);
 				for(uint32_t i = 0; i < count; i++)
 				{
-					property = objValue->getProperty(i);
-					static_cast<Array*>(asobj)->set(i, _MR(property->getASObject()));
-					delete property;
+					const ExtVariant& property = objValue->getProperty(i);
+					static_cast<Array*>(asobj)->set(i, _MR(property.getASObject(objectsMap)));
 				}
 			}
 			// We are converting an object, so lets set variables
 			else
 			{
 				asobj = Class<ASObject>::getInstanceS();
+				objectsMap[objValue] = asobj;
 			
 				ExtIdentifier** ids;
 				uint32_t count;
 				std::stringstream conv;
-				if(objValue != NULL && objValue->enumerate(&ids, &count))
+				if(objValue->enumerate(&ids, &count))
 				{
 					for(uint32_t i = 0; i < count; i++)
 					{
-						property = objValue->getProperty(*ids[i]);
+						const ExtVariant& property = objValue->getProperty(*ids[i]);
 
 						if(ids[i]->getType() == ExtIdentifier::EI_STRING)
 						{
 							asobj->setVariableByQName(ids[i]->getString(), "",
-									property->getASObject(), DYNAMIC_TRAIT);
+									property.getASObject(objectsMap), DYNAMIC_TRAIT);
 						}
 						else
 						{
 							conv.str("");
 							conv << ids[i]->getInt();
-							if(asobj->hasPropertyByMultiname(QName(conv.str(),""),true))
+							if(asobj->hasPropertyByMultiname(QName(conv.str(),""),true,true))
 							{
 								LOG(LOG_NOT_IMPLEMENTED,"ExtVariant::getASObject: duplicate property " << conv.str());
 								continue;
 							}
 							asobj->setVariableByQName(conv.str().c_str(), "",
-									property->getASObject(), DYNAMIC_TRAIT);
+									property.getASObject(objectsMap), DYNAMIC_TRAIT);
 						}
-						delete property;
 						delete ids[i];
 					}
 				}
-				delete ids;
+				delete[] ids;
 			}
-			if(objValue != NULL)
-				delete objValue;
 		}
 		break;
 	case EV_NULL:
@@ -356,6 +333,12 @@ ASObject* ExtVariant::getASObject() const
 }
 
 /* -- ExtASCallback -- */
+ExtASCallback::~ExtASCallback() {
+	func->decRef();
+	if(asArgs)
+		delete[] asArgs;
+}
+
 void ExtASCallback::call(const ExtScriptObject& so, const ExtIdentifier& id,
 		const ExtVariant** args, uint32_t argc, bool synchronous)
 {
@@ -369,10 +352,18 @@ void ExtASCallback::call(const ExtScriptObject& so, const ExtIdentifier& id,
 
 	// The caller indicated the VM isn't currently suspended,
 	// so add a FunctionEvent to the VM event queue.
+
+	// Convert ExtVariant arguments to ASObjects
+	assert(!asArgs);
+	asArgs = new ASObject*[argc];
+	std::map<const ExtObject*, ASObject*> objectsMap;
+	for(uint32_t i = 0; i < argc; i++)
+		asArgs[i] = args[i]->getASObject(objectsMap);
+
 	if(!synchronous)
 	{
 		func->incRef();
-		funcEvent = _MR(new (getSys()->unaccountedMemory) ExternalCallEvent(_MR(func), args, argc, &result, &exceptionThrown, &exception));
+		funcEvent = _MR(new (getSys()->unaccountedMemory) ExternalCallEvent(_MR(func), asArgs, argc, &result, &exceptionThrown, &exception));
 		// Add the callback function event to the VM event queue
 		funcWasCalled=getVm()->addEvent(NullRef,funcEvent);
 		if(!funcWasCalled)
@@ -383,16 +374,8 @@ void ExtASCallback::call(const ExtScriptObject& so, const ExtIdentifier& id,
 	{
 		try
 		{
-			// Convert ExtVariant arguments to ASObjects
-			ASObject** objArgs = g_newa(ASObject*,argc);
-			for(uint32_t i = 0; i < argc; i++)
-			{
-				objArgs[i] = args[i]->getASObject();
-			}
-
 			/* TODO: shouldn't we pass some global object instead of Null? */
-			ASObject* asObjResult = func->call(getSys()->getNullRef(), objArgs, argc);
-			result = new ExtVariant(_MR(asObjResult));
+			result = func->call(getSys()->getNullRef(), asArgs, argc);
 		}
 		// Catch AS exceptions and pass them on
 		catch(ASObject* _exception)
@@ -406,6 +389,8 @@ void ExtASCallback::call(const ExtScriptObject& so, const ExtIdentifier& id,
 			getSys()->setError(e.cause);
 		}
 		funcWasCalled = true;
+		delete[] asArgs;
+		asArgs = NULL;
 	}
 }
 void ExtASCallback::wait()
@@ -418,14 +403,18 @@ void ExtASCallback::wakeUp()
 	if(!funcEvent.isNull())
 		funcEvent->done.signal();
 }
-bool ExtASCallback::getResult(const ExtScriptObject& so, ExtVariant** _result)
+bool ExtASCallback::getResult(std::map<const ASObject*, std::unique_ptr<ExtObject>>& objectsMap,
+		const ExtScriptObject& so, const ExtVariant** _result)
 {
 	funcEvent = NullRef;
 	// Did the callback throw an AS exception?
 	if(exceptionThrown)
 	{
 		if(result != NULL)
-			delete result;
+		{
+			result->decRef();
+			result = NULL;
+		}
 
 		// Pass on the exception to the container through the script object
 		so.setException(exception.raw_buf());
@@ -441,7 +430,7 @@ bool ExtASCallback::getResult(const ExtScriptObject& so, ExtVariant** _result)
 	else if(result != NULL)
 	{
 		// Convert the result
-		*_result = result;
+		*_result = new ExtVariant(objectsMap, _MR(result));
 		success = true;
 	}
 	// No exception but also no result, still a success
@@ -452,6 +441,14 @@ bool ExtASCallback::getResult(const ExtScriptObject& so, ExtVariant** _result)
 	result = NULL;
 	exceptionThrown = false;
 	exception = "";
+	if (asArgs)
+	{
+		// The references have been consumed by the called
+		// function. We only delete the array.
+		delete[] asArgs;
+		asArgs = NULL;
+	}
+
 	return success;
 }
 /* -- ExtBuiltinCallback -- */
@@ -484,7 +481,8 @@ void ExtBuiltinCallback::wait()
 void ExtBuiltinCallback::wakeUp()
 {
 }
-bool ExtBuiltinCallback::getResult(const ExtScriptObject& so, ExtVariant** _result)
+bool ExtBuiltinCallback::getResult(std::map<const ASObject*, std::unique_ptr<ExtObject>>& objectsMap,
+		const ExtScriptObject& so, const ExtVariant** _result)
 {
 	// Set the result
 	*_result = result;
