@@ -17,8 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef _FLASH_ACCESSIBILITY_H
-#define _FLASH_ACCESSIBILITY_H
+#ifndef SCRIPTING_FLASH_ACCESSIBILITY_FLASHACCESSIBILITY_H
+#define SCRIPTING_FLASH_ACCESSIBILITY_FLASHACCESSIBILITY_H 1
 
 #include "asobject.h"
 
@@ -44,4 +44,4 @@ public:
 };
 
 }
-#endif
+#endif /* SCRIPTING_FLASH_ACCESSIBILITY_FLASHACCESSIBILITY_H */
