@@ -17,18 +17,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#include "abc.h"
-#include "class.h"
+#include "scripting/abc.h"
+#include "scripting/class.h"
 
-#include "flashdesktop.h"
+#include "scripting/flash/desktop/flashdesktop.h"
 
 using namespace std;
 using namespace lightspark;
 
-SET_NAMESPACE("flash.desktop");
-
-REGISTER_CLASS_NAME(NativeApplication);
-
 void NativeApplication::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
