@@ -17,8 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef _FLASH_XML_H
-#define _FLASH_XML_H
+#ifndef SCRIPTING_FLASH_XML_FLASHXML_H
+#define SCRIPTING_FLASH_XML_FLASHXML_H 1
 
 #include "compat.h"
 #include "asobject.h"
@@ -36,12 +36,14 @@ friend class XML;
 protected:
 	_NR<XMLDocument> root;
 	xmlpp::Node* node;
+	tiny_string toString_priv(xmlpp::Node *outputNode);
 public:
 	XMLNode(Class_base* c):ASObject(c),root(NullRef),node(NULL){}
 	XMLNode(Class_base* c, _R<XMLDocument> _r, xmlpp::Node* _n);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
+	tiny_string toString();
 	ASFUNCTION(_constructor);
 	ASFUNCTION(firstChild);
 	ASFUNCTION(childNodes);
@@ -49,6 +51,7 @@ public:
 	ASFUNCTION(_getNodeType);
 	ASFUNCTION(_getNodeName);
 	ASFUNCTION(_getNodeValue);
+	ASFUNCTION(_toString);
 };
 
 class XMLDocument: public XMLNode, public XMLBase
@@ -57,13 +60,15 @@ friend class XMLNode;
 private:
 	xmlpp::Node* rootNode;
 public:
-	XMLDocument(Class_base* c):XMLNode(c),rootNode(NULL){}
+	XMLDocument(Class_base* c, tiny_string s="");
 	void parseXMLImpl(const std::string& str);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
+	tiny_string toString();
 	ASFUNCTION(_constructor);
 	ASFUNCTION(parseXML);
 	ASFUNCTION(firstChild);
+	ASFUNCTION(_toString);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
@@ -71,4 +76,4 @@ public:
 };
 
 };
-#endif
+#endif /* SCRIPTING_FLASH_XML_FLASHXML */
